#include  "arduinoFFT.h"
#include <driver/i2s.h>
#include <WiFi.h>
#include <PubSubClient.h>
#include "config.h"
#define I2S_WS 11
#define I2S_SD 10
#define I2S_SCK 12
#define I2S_PORT I2S_NUM_0
#define bufferLen 64
#define nsample 128
#define sample_freq 8400  
#define TOPIC_FFT    TOPIC_PREFIX "/FFT"
arduinoFFT  FFT = arduinoFFT();


int16_t sBuffer[bufferLen];
WiFiClient wifiClient;
PubSubClient mqtt(MQTT_BROKER, 1883, wifiClient);

void i2s_install(){
  const i2s_config_t i2s_config = {
    .mode = i2s_mode_t(I2S_MODE_MASTER | I2S_MODE_RX),
    .sample_rate = sample_freq*2,
    .bits_per_sample = i2s_bits_per_sample_t(16),
    .channel_format = I2S_CHANNEL_FMT_ONLY_LEFT,
    .communication_format = i2s_comm_format_t(I2S_COMM_FORMAT_STAND_I2S),
    .intr_alloc_flags = 0, // default interrupt priority
    .dma_buf_count = 8,
    .dma_buf_len = bufferLen,
    .use_apll = false
  };

  i2s_driver_install(I2S_PORT, &i2s_config, 0, NULL);
}
void i2s_setpin(){
  const i2s_pin_config_t pin_config = {
    .bck_io_num = I2S_SCK,
    .ws_io_num = I2S_WS,
    .data_out_num = -1,
    .data_in_num = I2S_SD
  };
  i2s_set_pin(I2S_PORT, &pin_config);
}



void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
    printf("\nWiFi connected.\n");
}
void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
}




void setup() {
  connect_wifi();
  connect_mqtt();


  Serial.begin(115200);
  Serial.println("Setup I2S ...");
  
  delay(1000);
  i2s_install();
  i2s_setpin();
  i2s_start(I2S_PORT);
  delay(500);
}
void loop() {
  double samples[nsample];
  double img[nsample];
  for (int j=0;j<nsample;j+=bufferLen/4)
  {
          size_t bytesIn = 0;
          esp_err_t result = i2s_read(I2S_PORT, &sBuffer, bufferLen, &bytesIn, portMAX_DELAY);
          
          if (result == ESP_OK)
          {
                int samples_read = bytesIn / 2;
                if (samples_read > 0)
                {

                      for (int i = 0; i < samples_read/2; ++i) 
                      {
                        samples[i+j]=sBuffer[2*i+1];
                        img[i+j]=0;
                      }
                }
          }
  }
  
    /*Perform FFT on samples*/
    FFT.Windowing(samples, nsample, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
    FFT.Compute(samples,img  , nsample, FFT_FORWARD);
    FFT.ComplexToMagnitude(samples, img, nsample);
    /*Find peak frequency and print peak*/
    double peak = FFT.MajorPeak(samples,  nsample, sample_freq);
    Serial.println(peak);     //Print out the most  dominant frequency.
    String payload(peak);
    mqtt.publish(TOPIC_FFT, payload.c_str());
    delay(800);
}
