###########################################
# -------------- [ IMPORTS ] --------------
###########################################

from machine import PWM, Pin
from time import sleep, ticks_ms
import CONNECT, TOPIC


###########################################
# --------------- [ SETUP ] ---------------
###########################################

buzzer_Pin = 4
buzzer = PWM(Pin(buzzer_Pin, Pin.OUT), duty=0)
VOLUME = 25


notes = {
    'A0': 27, 'A#0': 29, 'Bb0': 29, 'B0': 31,
    'C1': 33, 'C#1': 35, 'Db1': 35, 'D1': 37, 'D#1': 39, 'Eb1': 39, 'E1': 41, 'F1': 44, 'F#1': 46, 'Gb1': 46, 'G1': 49, 'G#1': 52, 'Ab1': 52,
    'A1': 55, 'A#1': 58, 'Bb1': 58, 'B1': 62,
    'C2': 65, 'C#2': 69, 'Db2': 69, 'D2': 73, 'D#2': 78, 'Eb2': 78, 'E2': 82, 'F2': 87, 'F#2': 92, 'Gb2': 92, 'G2': 98, 'G#2': 104, 'Ab2': 104,
    'A2': 110, 'A#2': 117, 'Bb2': 117, 'B2': 123,
    'C3': 131, 'C#3': 139, 'Db3': 139, 'D3': 147, 'D#3': 156, 'Eb3': 156, 'E3': 165, 'F3': 175, 'F#3': 185, 'Gb3': 185, 'G3': 196, 'G#3': 208, 'Ab3': 208,
    'A3': 220, 'A#3': 233, 'Bb3': 233, 'B3': 247,
    'C4': 262, 'C#4': 277, 'Db4': 277, 'D4': 294, 'D#4': 311, 'Eb4': 311, 'E4': 330, 'F4': 349, 'F#4': 370, 'Gb4': 370, 'G4': 392, 'G#4': 415, 'Ab4': 415,
    'A4': 440, 'A#4': 466, 'Bb4': 466, 'B4': 494,
    'C5': 523, 'C#5': 554, 'Db5': 554, 'D5': 587, 'D#5': 622, 'Eb5': 622, 'E5': 659, 'F5': 698, 'F#5': 740, 'Gb5': 740, 'G5': 784, 'G#5': 831, 'Ab5': 831,
    'A5': 880, 'A#5': 932, 'Bb5': 932, 'B5': 988,
    'C6': 1047, 'C#6': 1109, 'Db6': 1109, 'D6': 1175, 'D#6': 1245, 'Eb6': 1245, 'E6': 1319, 'F6': 1397, 'F#6': 1480, 'Gb6': 1480, 'G6': 1568, 'G#6': 1661, 'Ab6': 1661,
    'A6': 1760, 'A#6': 1865, 'Bb6': 1865, 'B6': 1976,
    'C7': 2093, 'C#7': 2217, 'Db7': 2217, 'D7': 2349, 'D#7': 2489, 'Eb7': 2489, 'E7': 2637, 'F7': 2794, 'F#7': 2960, 'Gb7': 2960, 'G7': 3136, 'G#7': 3322, 'Ab7': 3322,
    'A7': 3520, 'A#7': 3729, 'Bb7': 3729, 'B7': 3951,
    'C8': 4186, 'C#8': 4435, 'Db8': 4435, 'D8': 4699, 'D#8': 4978, 'Eb8': 4978, 'E8': 5274, 'F8':5588, 'F#8': 5920, 'Gb8': 5920, 'G8': 6272, 'G#8': 6645, 'Ab8': 6645,
    'A8': 7040, 'A#8': 7459, 'Bb8': 7459, 'B8': 7902
    }

fftfreq = sorted(notes.items(), key = lambda x : x[1], reverse = False)

###########################################
# ------------- [ FUNCTIONS ] -------------
###########################################

def set_vol(n=0): buzzer.duty(int(n/100*1023))
def set_note(n='A4'):
    global notes
    buzzer.freq(notes[n[0].upper()+n[1:]])

def playmusic(song_dict, vol=VOLUME):
    
    # REQUIREMENT:
    # In song dict, you must have:
    # 1. 'song' : list()
    # 2. 'note_value : float() e.g. 4 = whole note, 1/2 = half note
    # 3. 'tempo' : int() e.g. 80 means 80 beats per minute (bpm)
    
    note_value = song_dict['note_value']
    song = song_dict['song']
    tempo = song_dict['tempo']
    set_vol(vol)
    
    for i in song:
        CONNECT.mqtt.check_msg()
        if TOPIC.song_stat%2==0 or TOPIC.mode_stat!='4':
            set_vol(0)
            return False
        else:
            if i==0: set_vol(0)
            else:
                set_note(i)
                set_vol(vol)
            sleep(note_value/tempo*60)
    set_vol(0)
    return True
    
def get_song(n): return playmusic(songs[n]['var'])
    
def metronome(bpm, note='A5', vol=VOLUME):
    tick = ticks_ms()/1000
    play = True
    set_vol(0)
    while bpm>=0:
        CONNECT.mqtt.check_msg()
        if TOPIC.metro_stat%2==0 or TOPIC.mode_stat!='3':
            print("[MODE 3] METRO | Break")
            set_vol(0)
            break
        if play:
            now = ticks_ms()/1000 # bpm 60: every 1 second = 1 tick
            try:
                bpm = TOPIC.metro_bpm
                if now - tick >= 60/float(bpm):
                    set_note(note)
                    set_vol(vol)
                    play = False
                    tick = now
            except: pass
        else:
            now = ticks_ms()/1000
            if now - tick >= 0.05:
                set_vol(0)
                play = True
    set_vol(0)
    
                
###########################################         
# --------------- [ SONGS ] ---------------
###########################################

kasetsart = {'note_value': 1/2,
             'tempo': 100,
             'song': ["A4","A4","A4","A4","A4",0,"F#4","A4","C5",0,"A4","A4","A4","A4","A4",0,
                      "A4","A4","A4","A4","A4",0,"F#4","A4","C#5",0,"A4","A4","A4","A4","A4",0,"C#4","D4","D#4","E4","A4","A4",0,0,
            "A4","A4","A4","A4","F#4","F#4","D4","D4","E4","E4","E4","E4",0,"E4","F#4","G4","A4","A4","C#5","C#5","C#5","C#5","A4","A4",
            "F#4","F#4","F#4","F#4","F#4","F#4","F#4",0,"D5","D5","D5","D5","B4","B4","G4","G4","C#5","C#5","C#5","C#5","A4","A4",
            "C#4","C#4","A4","A4","A4","A4","G4","G4","G4","G4","F#4","F#4","F#4","F#4","F#4","F#4","F#4",0,
            "D5","D5","D5","D5","B4","B4","F#4","F#4","A4","A4","A4","A4",0,"E4","F#4","G4","A4","A4","C#5",0,"C#5","C#5","A4","A4",
            "E5","E5","D5","D5","C#5","C#5","B4","B4","D5","D5","D5","D5","B4","B4","C#4","C#4","B4","B4","B4","B4","A4","A4","C#4","C#4",
            "D4","D4","D4","D4","D4","D4","D4","D4","D4","D4","D4",0,"F#4","F#4","D4","D4","E4","E4","E4","E4","F#4","F#4","G4","G4",
            "E4","E4","F#4","F#4","G4","G4","A4","A4","B4","B4","B4","C#4","D4","D4","D4","E4","F#4","F#4","F#4","F#4","F#4","F#4",0,"G4","F#4",
            "E4","E4","E4","E4","F#4","F#4","G4","G4","E4","E4","F#4","F#4","G4","G4","G#4","G#4","A4",0,"A4",0,"A4",0,"A4",0,"A4","A4","A4","A4","A4","A4",
            "B4","C#5","D5","D5","D5","D5","B4","B4","F#4","F#4","A4","A4","A4","A4",0,"E4","F#4","G4","A4","A4","C#5",0,"C#5","C#5","A4","A4",
            "E5","E5","D5","D5","C#5","C#5","B4","B4","D5","D5","D5","D5","B4","B4","C#4","C#4","B4","B4","B4","B4","A4","A4","C#4","C#4",
            "D4","D4","D4","D4","D4","D4","D4","D4"]}

mario = {'note_value': 1/4,
         'tempo': 120,
         'song': ["E7", "E7", 0, "E7", 0, "C7", "E7", 0, "G7", 0, 0, 0, "G6", 0, 0, 0, "C7", 0, 0, "G6",
         0, 0, "E6", 0, 0, "A6", 0, "B6", 0, "A#6", "A6", 0, "G6", "E7", 0, "G7", "A7", 0, "F7", "G7",
         0, "E7", 0,"C7", "D7", "B6", 0, 0, "C7", 0, 0, "G6", 0, 0, "E6", 0, 0, "A6", 0, "B6", 0,
         "A#6", "A6", 0, "G6", "E7", 0, "G7", "A7", 0, "F7", "G7", 0, "E7", 0,"C7", "D7", "B6", 0, 0]}

nokia = {'note_value': 1/2,
         'tempo': 180,
          'song': ["E5", "D#5", "F#4", "F#4", "G#4", "G#4", "C#5", "B4", "D4", "D4", "E4", "E4", "B4", "A4", "C#4", "C#4", "E4", "E4", "A4", "A4", "A4", "A4", 0,]}

icecream = {'note_value': 1/2,
             'tempo': 120,
             'song': ["E5", "E5", "G5", "C5", "C5", "C5", 0, 0, "A5", "A5", "C6", "F5", "F5", "F5", "F5", 0, "B5", "B5", "C6", "D6", "C6", "C6", "C6", 0,
                  "E5", "E5", "G5", "C5", "C5", "C5", 0, 0, "A5", "A5", "C6", "F5", "F5", 0, "A5", "B5", "G5", "A5", "B5", "D6", "C6", "C6", "C6"]}

doraemon_intro = {'note_value': 1/2,
             'tempo': 180,
             'song': ["D5", "C#5", "B4", "A4", "B4", "C#5", "D5", "C#5", "B4", "A4", "B4", "C#5", "D5", "C#5", "B4", "A4", "B4", "C#5", "D5", "C#5", "B4", "A4", "A4", 0,
                      "E5", "D5", "C#5", "B4", "C#5", "D5", "E5", "D5", "C#5", "B4", "C#5", "D5", "E5", "D5", "C#5", "B4", "C#5", "D5", "C#5", "D5", "D#5", "E5", "E5", 0,
                      "B5", "C#6", "D6", "G5", "A5", "B5", "A5", "B5", "C#6", "F#5", "G5", "A5", "G5", "A5", "B5", "E5", "F#5", "G5", "F#5", "G5", "A5", "D5", "E5", "F#5",
                      "E5", "Eb5", "D5", "C#5", "C5", "B4", "Bb4", "A4", "Ab4", "G4", "F#4", "F4", "E4"]}

songs = {
        '0': {'name':'','var':''},
        '1': {'name':"Kasetsart", 'var':kasetsart},
        '2':{'name':'Mario', 'var':mario},
        '3':{'name':'Nokia', 'var':nokia},
        '4':{'name':'Ice-cream', 'var': icecream},
        '5':{'name':'Doraemon', 'var':doraemon_intro}
         }