from machine import Pin, I2C, SoftI2C
import ssd1306, MUSIC
from math import sqrt, sin, cos, radians

i2c = SoftI2C(sda=Pin(48), scl=Pin(47))
display = ssd1306.SSD1306_I2C(128, 64, i2c)

def main(mode_num, payload, reset=False):
    
    # MODE 1 : Note Detector (Tuner)
    # MODE 2 : Note Display
    # MODE 3 : Metronome
    # MODE 4 : Sing a song
    
    m = mode_num
    display.fill(0)
    box(m)
    
    if not reset:
        if m=='1': MODE_1(payload)
        elif m=='2': MODE_2(payload)
        elif m=='3': MODE_3(payload)
        elif m=='4': MODE_4(payload)
    
    display.show()
    
def MODE_1(p): MODE_2(p)

def MODE_2(p):
    
    # Payload format: f"{ letter }{ accidental (opt.) }{ number }"

    if len(p)==2: p = p[0] + 'n' + p[1]
    if len(p)==3:
        letter_handle(p[0])
        accidental_handle(p[1])
        number_handle(p[2])
    
def MODE_3(p):
    # Payload format: '120', '150', '80'
    display.text(p+' BPM',find_mid(p+' BPM'),int((60+15)/2),1)

def MODE_4(p):
    # Payload format: (Song ID) '1', '2'
    song_name = MUSIC.songs[p]['name']
    display.text('Playing :',find_mid('Playing :'),30,1)
    display.text(str(song_name),find_mid(str(song_name)),50,1)

def box(mode):
    display.rect(0,0,128,15,1)
    if mode=='1': text = "MODE 1 : TUNER"
    elif mode=='2': text = "MODE 2 : N.DISP"
    elif mode=='3': text = "MODE 3 : METRO"
    elif mode=='4': text = "MODE 4 : SONG"
    else: text = ''
    display.text(text,5,4,1)

def column(i):
    # Returns the x_set
    if i==1: return 0
    if i==2: return 30+10
    if i==3: return 70

def hline(x,y,length=30, col=1):
    set_x = 13+column(col)
    set_y = 23
    for i in range(3): display.hline(x+set_x, y+set_y+i, length, 1)

def vline(x,y,length=40, col=1):
    set_x = 13+column(col)
    set_y = 23
    for i in range(3): display.vline(x+set_x+i, y+set_y, length, 1)
    
def lineL(x,y,length,angle,col=1):
    set_x = 13+column(col)+1
    set_y = 23+1
    x1 = int(x+set_x)
    y1 = int(y+set_y)
    x2 = int(x1+length*cos(-radians(angle+180)))
    y2 = int(y1+length*sin(-radians(angle+180)))
    for i in range(3): display.line(x1+i, y1, x2+i, y2, 1)
     
def lineR(x,y,length,angle,col=1):
    set_x = 13+column(col)+1
    set_y = 23+1
    x1 = int(x+set_x)
    y1 = int(y+set_y)
    x2 = int(x1+length*cos(radians(angle)))
    y2 = int(y1+length*sin(radians(angle)))
    for i in range(3): display.line(x1, y1+i, x2, y2+i, 1)
    
def find_mid(text): return int((128-len(text)*6.5)/2)
        
def letter_handle(t):
    t = t.lower()
    if t=='a': A()
    elif t=='b': B()
    elif t=='c': C()
    elif t=='d': D()
    elif t=='e': E()
    elif t=='f': F()
    elif t=='g': G()
    
def number_handle(t):
    if t=='0': ZERO()
    elif t=='1': ONE()
    elif t=='2': TWO()
    elif t=='3': THREE()
    elif t=='4': FOUR()
    elif t=='5': FIVE()
    elif t=='6': SIX()
    elif t=='7': SEVEN()
    elif t=='8': EIGHT()
    
def accidental_handle(t):
    if t=='#': SHARP()
    elif t=='b': FLAT()
    elif t=='n': NATURAL()

def A():
    lineL(15,0,100,70)
    lineL(15,0,100,110)
    for i in range(3): display.hline(13+8,23+25+i,18,1)
    
def B():
    vline(0,0)
    hline(0,0,20)
    lineR(19,0,200**0.5,45)
    lineL(28,11,200**0.5,45)
    hline(0,21,21)
    lineR(19,21,200**0.5,45)
    lineL(28,32,200**0.5,45)
    hline(0,38,25)    

def C():
    vline(0,0)
    hline(0,0)
    hline(0,40-3)

def D():
    vline(0,0)
    hline(0,0,20)
    lineL(29,20,10*sqrt(5),-63)
    lineL(20,38,10*sqrt(5)-2,180+63)
    hline(0,38,22) 
    
def E():
    hline(0,0)
    vline(0,0)
    hline(0,20-3)
    hline(0,40-3)

def F():
    hline(0,0)
    vline(0,0)
    hline(0,20-3)
   
def G():
    vline(0,0)
    hline(0,0)
    hline(0,40-3)
    hline(15,20,15)
    vline(27,20,20)

def ZERO():
    vline(0,0,col=3)
    hline(0,0,col=3)
    hline(0,37,col=3)
    vline(27,0,col=3)
    
def ONE():
    lineL(12,0,20,50,col=3)
    vline(14,0,col=3)
    hline(0,37,col=3)
    
def TWO():
    hline(0,0,col=3)
    hline(0,17,col=3)
    hline(0,37,col=3)
    vline(27,0,20,col=3)
    vline(0,17,20,col=3)

def THREE():
    hline(0,0,col=3)
    hline(0,17,col=3)
    hline(0,37,col=3)
    vline(27,0,col=3)
    
def FOUR():
    lineL(18,0,35,58,col=3)
    vline(20,0,col=3)
    hline(0,30,col=3)

def FIVE():
    hline(0,0,col=3)
    hline(0,17,col=3)
    hline(0,37,col=3)
    vline(0,0,20,col=3)
    vline(27,17,20,col=3)

def SIX():
    hline(0,0,col=3)
    hline(0,17,col=3)
    hline(0,37,col=3)
    vline(0,0,col=3)
    vline(27,17,20,col=3)

def SEVEN():
    hline(0,0,col=3)
    lineL(27,0,50,60,col=3)

def EIGHT():
    hline(0,0,col=3)
    hline(0,17,col=3)
    hline(0,37,col=3)
    vline(0,0,col=3)
    vline(27,0,col=3)
    
def SHARP():
    hline(0,5,20,col=2)
    hline(0,12,20,col=2)
    vline(5,0,20,col=2)
    vline(12,0,20,col=2)
    
def NATURAL():
    vline(0,0,20,col=2)
    hline(0,10,10,col=2)
    hline(0,20,10,col=2)
    vline(10,10,20,col=2)
    
def FLAT():
    vline(0,0,25,col=2)
    hline(0,10,13,col=2)
    lineL(9,10,16,60,2)