###########################################
# -------------- [ IMPORTS ] --------------
###########################################

import MUSIC, OLED, CONNECT

TOPIC_PREFIX = 'b6610505314'

###########################################
# -------------- [ TOPICS ] --------------
###########################################

TOPIC_INPUT_NOTE = f'{TOPIC_PREFIX}/input/note'
TOPIC_INPUT_METRO_STAT = f'{TOPIC_PREFIX}/input/metro/stat'
TOPIC_INPUT_METRO_BPM = f'{TOPIC_PREFIX}/input/metro/bpm'
TOPIC_INPUT_MODE = f'{TOPIC_PREFIX}/input/mode'
TOPIC_INPUT_SONG_NUM = f'{TOPIC_PREFIX}/input/song/num'
TOPIC_INPUT_SONG_STAT = f'{TOPIC_PREFIX}/input/song/stat'
TOPIC_INPUT_FFT = f'{TOPIC_PREFIX}/input/frequency'
TOPIC_OUTPUT_FFT = f'{TOPIC_PREFIX}/output/frequency'
TOPIC_OUTPUT_FFT_ERROR = f'{TOPIC_PREFIX}/output/error'


metro_bpm, metro_stat, song_stat, mode_stat, song_id = 0,0,0,0,'0'

def subscribe_topics():
    CONNECT.mqtt.subscribe(TOPIC_INPUT_NOTE)
    CONNECT.mqtt.subscribe(TOPIC_INPUT_METRO_STAT)
    CONNECT.mqtt.subscribe(TOPIC_INPUT_METRO_BPM)
    CONNECT.mqtt.subscribe(TOPIC_INPUT_MODE)
    CONNECT.mqtt.subscribe(TOPIC_INPUT_SONG_NUM)
    CONNECT.mqtt.subscribe(TOPIC_INPUT_SONG_STAT)
    CONNECT.mqtt.subscribe(TOPIC_INPUT_FFT)

 
def mqtt_callback(topic, payload): # Receive Values
    global mode_stat
    t, p = str(topic.decode()), str(payload.decode())
    
    if t == TOPIC_INPUT_MODE: INPUT_MODE(p)
      
    if mode_stat == '1':
        if t == TOPIC_INPUT_FFT: PROCESS_FFT(mode_stat, p)
    
    elif mode_stat == '2':
        if t == TOPIC_INPUT_NOTE: INPUT_NOTE(mode_stat, p)
        
    elif mode_stat == '3':
        if t == TOPIC_INPUT_METRO_STAT: INPUT_METRO_STAT(mode_stat, p)
        elif t == TOPIC_INPUT_METRO_BPM: INPUT_METRO_BPM(mode_stat, p)
    
    elif mode_stat == '4':
        if t == TOPIC_INPUT_SONG_NUM: INPUT_SONG_NUM(mode_stat, p)
        elif t == TOPIC_INPUT_SONG_STAT: INPUT_SONG_STAT(mode_stat, p)
            
            

###########################################
# ------------- [ FUNCTIONS ] -------------
###########################################
    
def INPUT_MODE(p):
    global mode_stat
    mode_stat = p
    MUSIC.set_vol(0)
    OLED.main(mode_stat, p, reset=True)
    print(f"[MODE]: {p}")
    
def PROCESS_FFT(mode, p):
    notelist = MUSIC.fftfreq
    p = int(float(p))
    for i in range(len(notelist)):
        if(notelist[i][1] >= p):
            try: notepub, error = (notelist[i-1][0], (notelist[i-1][1] - p)) if (notelist[i][1] + notelist[i-1][1])/2 >= p else (notelist[i][0], (notelist[i][1] - p))     
            except: notepub, error = notelist[i][0], notelist[i][1] - p
            break
    try:
        CONNECT.mqtt.publish(TOPIC_OUTPUT_FFT, notepub)
        CONNECT.mqtt.publish(TOPIC_OUTPUT_FFT_ERROR, str(error))
        OLED.main(mode, notepub)
    except: pass
    print(f"[MODE 1] Tuner {notepub} {error}")
    
def INPUT_SONG_NUM(mode, p):
    global song_id
    song_id = p
    print(f"[MODE 4] Song ID : {p}")
    
def INPUT_SONG_STAT(mode, p):
    global song_stat, song_id
    if song_id=='0': return
    song_stat += int(p)
    if song_stat%2!=0:
        print(f"[MODE {mode}] Status | PLAY")
        OLED.main(mode, song_id)
        if MUSIC.get_song(song_id):
            song_stat+=1
            OLED.main(mode, '0', reset=True)
    else:
        OLED.main(mode, '0', reset=True)
        print(f"[MODE {mode}] Status | STOP")

def INPUT_METRO_BPM(mode, p):
    global metro_bpm
    
    bpm = metro_bpm
    error = False
    
    # check input
    try: bpm = float(p)
    except: error = True
    if float(bpm)<0: error = True
    else: metro_bpm = bpm

    # main
    if error: print(f"[MODE {mode}] BPM | Error IN: {p}")
    else:
        OLED.main(mode, p)
        print(f"[MODE {mode}] BPM | {metro_bpm}")

def INPUT_METRO_STAT(mode, p):
    global metro_stat, metro_bpm
    metro_stat += int(p)
    if metro_stat%2!=0:
        print(f"[MODE {mode}] Status | PLAY")
        MUSIC.metronome(metro_bpm)
    else: print(f"[MODE {mode}] Status | STOP")
        
def INPUT_NOTE(mode, p):
    
    error = True
    note = ''
    
    # fix input
    if len(p)==2: note = p[0].upper()+'n'+p[1]
    else: note = p
    
    # check input (note)
    lowest = []
    highest = ['Cn8']
    letter = ['A','B','C','D','E','F','G']
    accidental = ['#', 'b', 'n']
    number = ['2','3','4','5','6','7','8']
    
    if p == '': error = False
    elif note[0] in letter and\
       note[1] in accidental and\
       note[2] in number and\
       len(p) in [2, 3]: error = False
    elif note in highest+lowest: error = False
    
    # main
    if error: print(f"[MODE {mode}] Error IN: {p}")
    else:
        if p == '':
            MUSIC.set_vol(0)
            print(f'[MODE {mode}] STOP')
        else:
            MUSIC.set_vol(50)
            MUSIC.set_note(p)
            print(f"[MODE {mode}] Playing: {p}")
        OLED.main(mode, note)